function [ A_symmetry ] = A_symmetry_fun(m_regr,combos_all_unique)

%% Impose symmetry wrt s (i.e. delta) for any given vector of prices
% NB: I can use combos_all as an input the way I do below because the
% columns of combos_all are ordereed so that the first one moves faster,
% the second one a bit slower, etc.

J = 2;

combos_s_p = unique(combos_all_unique(:,1:2*J),'rows');
n_Bcoeff_s_p = size(combos_s_p,1);

A_symmetry = [];

parfor h=1:m_regr(6)+1           % for any price_outside
    for k=1:m_regr(5)+1           % for any income
        for w=1:n_Bcoeff_s_p           % combos for s and p
            for j=2:J                 % good wrt which we take the increment
                
                ind_reorder = [j setdiff(1:J,j)];
                [~,position] = sort(ind_reorder);     % position(r) gives the position of the r-th good in the indices vector
                
                combos_s = double(combos_s_p(w,1:J));
                combos_p = double(combos_s_p(w,J+1:2*J));
                temp_ind = [double(combos_s_p(w,:)) k h];
                
                combos_s_reord = combos_s(ind_reorder);
                combos_p_reord = combos_p(ind_reorder);
                temp_ind_reord = [rep2unique([combos_s_reord combos_p_reord],J) k h];
                
                %%
                
                if combos_s(j)<m_regr(2)+1 && combos_s(1)<m_regr(1)+1
                    
                    temp = zeros(1,size(combos_all_unique,1));
                    
                    % derivative of own inverse sigma wrt s_j
                    
                    combos_s_plus1 = combos_s + [zeros(1,j-1) 1 zeros(1,J-j)];
                    temp_ind2 = rep2unique([combos_s_plus1 combos_p],J);
                    temp_ind3 = [temp_ind2 k h];
                    
                    temp(ismember(combos_all_unique,temp_ind,'rows')) = temp(ismember(combos_all_unique,temp_ind,'rows')) - 1;
                    temp(ismember(combos_all_unique,temp_ind3,'rows')) = temp(ismember(combos_all_unique,temp_ind3,'rows')) + 1;
                    
                    % derivative of j inverse sigma wrt own
                    
                    vec = zeros(1,J);
                    vec(position(1)) = 1;
                    combos_s_plus1 = combos_s_reord + vec;
                    temp_ind2 = rep2unique([combos_s_plus1 combos_p_reord],J);
                    temp_ind3 = [temp_ind2 k h];
                    
                    temp(ismember(combos_all_unique,temp_ind_reord,'rows')) = temp(ismember(combos_all_unique,temp_ind_reord,'rows')) + 1;
                    temp(ismember(combos_all_unique,temp_ind3,'rows')) = temp(ismember(combos_all_unique,temp_ind3,'rows')) - 1;
                    
                    A_symmetry = [A_symmetry;
                        reshape(temp,1,numel(temp))];
                    
                end
                
                %%
                
                if combos_s(j)>1 && combos_s(1)>1
                    
                    temp = zeros(1,size(combos_all_unique,1));
                    
                    % derivative of own inverse sigma wrt s_j
                    
                    combos_s_minus1 = combos_s + [zeros(1,j-1) -1 zeros(1,J-j)];
                    temp_ind2 = rep2unique([combos_s_minus1 combos_p],J);
                    temp_ind3 = [temp_ind2 k h];
                    
                    temp(ismember(combos_all_unique,temp_ind,'rows')) = temp(ismember(combos_all_unique,temp_ind,'rows')) + 1;
                    temp(ismember(combos_all_unique,temp_ind3,'rows')) = temp(ismember(combos_all_unique,temp_ind3,'rows')) - 1;
                    
                    % derivative of j inverse sigma wrt own
                    
                    vec = zeros(1,J);
                    vec(position(1)) = 1;
                    combos_s_minus1 = combos_s_reord - vec;
                    temp_ind2 = rep2unique([combos_s_minus1 combos_p_reord],J);
                    temp_ind3 = [temp_ind2 k h];
                    
                    temp(ismember(combos_all_unique,temp_ind_reord,'rows')) = temp(ismember(combos_all_unique,temp_ind_reord,'rows')) - 1;
                    temp(ismember(combos_all_unique,temp_ind3,'rows')) = temp(ismember(combos_all_unique,temp_ind3,'rows')) + 1;
                    
                    A_symmetry = [A_symmetry;
                        reshape(temp,1,numel(temp))];
                    
                end
                
                %%
                
                if combos_s(j)>1 && combos_s(1)<m_regr(1)+1
                    
                    temp = zeros(1,size(combos_all_unique,1));
                    
                    % derivative of own inverse sigma wrt s_j
                    
                    combos_s_minus1 = combos_s + [zeros(1,j-1) -1 zeros(1,J-j)];
                    temp_ind2 = rep2unique([combos_s_minus1 combos_p],J);
                    temp_ind3 = [temp_ind2 k h];
                    
                    temp(ismember(combos_all_unique,temp_ind,'rows')) = temp(ismember(combos_all_unique,temp_ind,'rows')) + 1;
                    temp(ismember(combos_all_unique,temp_ind3,'rows')) = temp(ismember(combos_all_unique,temp_ind3,'rows')) - 1;
                    
                    % derivative of j inverse sigma wrt own
                    
                    vec = zeros(1,J);
                    vec(position(1)) = 1;
                    combos_s_plus1 = combos_s_reord + vec;
                    temp_ind2 = rep2unique([combos_s_plus1 combos_p_reord],J);
                    temp_ind3 = [temp_ind2 k h];
                    
                    temp(ismember(combos_all_unique,temp_ind_reord,'rows')) = temp(ismember(combos_all_unique,temp_ind_reord,'rows')) + 1;
                    temp(ismember(combos_all_unique,temp_ind3,'rows')) = temp(ismember(combos_all_unique,temp_ind3,'rows')) - 1;
                    
                    A_symmetry = [A_symmetry;
                        reshape(temp,1,numel(temp))];
                    
                end
                
                %%
                
                if combos_s(j)<m_regr(2)+1 && combos_s(1)>1
                    
                    temp = zeros(1,size(combos_all_unique,1));
                    
                    % derivative of own inverse sigma wrt s_j
                    
                    combos_s_plus1 = combos_s + [zeros(1,j-1) 1 zeros(1,J-j)];
                    temp_ind2 = rep2unique([combos_s_plus1 combos_p],J);
                    temp_ind3 = [temp_ind2 k h];
                    
                    temp(ismember(combos_all_unique,temp_ind,'rows')) = temp(ismember(combos_all_unique,temp_ind,'rows')) - 1;
                    temp(ismember(combos_all_unique,temp_ind3,'rows')) = temp(ismember(combos_all_unique,temp_ind3,'rows')) + 1;
                    
                    % derivative of j inverse sigma wrt own
                    
                    vec = zeros(1,J);
                    vec(position(1)) = 1;
                    combos_s_minus1 = combos_s_reord - vec;
                    temp_ind2 = rep2unique([combos_s_minus1 combos_p_reord],J);
                    temp_ind3 = [temp_ind2 k h];
                    
                    temp(ismember(combos_all_unique,temp_ind_reord,'rows')) = temp(ismember(combos_all_unique,temp_ind_reord,'rows')) - 1;
                    temp(ismember(combos_all_unique,temp_ind3,'rows')) = temp(ismember(combos_all_unique,temp_ind3,'rows')) + 1;
                    
                    A_symmetry = [A_symmetry;
                        reshape(temp,1,numel(temp))];
                    
                end
            end
            %k
        end
    end
end

A_symmetry = unique(A_symmetry,'rows');
A_symmetry = setdiff(A_symmetry,zeros(1,size(A_symmetry,2)),'rows');

end