function [ grad ] = deriv_BCS_fun_cross_p_grad(p_own,p_other,x_outf,x_usda,income,p_out,csi_demeaned,theta_BCS,good)

% NB: normal_draw are the draws from bivariate N(0,1) used to generate
% rc_draw

constant = theta_BCS(1:2);
coeff_p_income = theta_BCS(3);
coeff_p_out = theta_BCS(4);
coeff_x_outf = theta_BCS(5);
coeff_x_usda = theta_BCS(6);
coeff_p = theta_BCS(7:8);
weight = theta_BCS(9);

indices = [1 2];    % used below in gardient wrt constants

if good==2      % flip order when looking at good 2
    
    constant = constant([2 1]);
    indices = indices([2 1]);
    
end

p_own = reshape(p_own,[numel(p_own) 1]);
lp = length(p_own);

csi_own = csi_demeaned(1);
csi_other = csi_demeaned(2);

if good==1
    
    util_own_1 = constant(1) + coeff_p(1)*p_own + coeff_x_outf*x_outf + coeff_p_income*p_own.*income + coeff_p_out*p_out + csi_own;
    util_own_2 = constant(1) + coeff_p(2)*p_own + coeff_x_outf*x_outf + coeff_p_income*p_own.*income + coeff_p_out*p_out + csi_own;
    
    util_other_1 = constant(2) + coeff_p(1)*p_other + coeff_x_outf*x_outf + coeff_x_usda*x_usda + coeff_p_income*p_other.*income + coeff_p_out*p_out + csi_other;
    util_other_2 = constant(2) + coeff_p(2)*p_other + coeff_x_outf*x_outf + coeff_x_usda*x_usda + coeff_p_income*p_other.*income + coeff_p_out*p_out + csi_other;
    
elseif good==2
    
    util_own_1 = constant(1) + coeff_p(1)*p_own + coeff_x_outf*x_outf + coeff_x_usda*x_usda + coeff_p_income*p_own.*income + coeff_p_out*p_out + csi_own;
    util_own_2 = constant(1) + coeff_p(2)*p_own + coeff_x_outf*x_outf + coeff_x_usda*x_usda + coeff_p_income*p_own.*income + coeff_p_out*p_out + csi_own;
    
    util_other_1 = constant(2) + coeff_p(1)*p_other + coeff_x_outf*x_outf + coeff_p_income*p_other.*income + coeff_p_out*p_out + csi_other;
    util_other_2 = constant(2) + coeff_p(2)*p_other + coeff_x_outf*x_outf + coeff_p_income*p_other.*income + coeff_p_out*p_out + csi_other;
    
end

temp_own_1 = exp(util_own_1);
temp_own_2 = exp(util_own_2);

temp_other_1 = exp(util_other_1);
temp_other_2 = exp(util_other_2);

share_own_1 = temp_own_1./(1+temp_own_1+temp_other_1);
share_own_2 = temp_own_2./(1+temp_own_2+temp_other_2);

share_other_1 = temp_other_1./(1+temp_own_1+temp_other_1);
share_other_2 = temp_other_2./(1+temp_own_2+temp_other_2);



grad = zeros(length(theta_BCS),1);

% own constant
grad(indices(1)) = -weight*(coeff_p(1) + coeff_p_income*income)*share_own_1*share_other_1*(1-2*share_own_1) - ...
          (1-weight)*(coeff_p(2) + coeff_p_income*income)*share_own_2*share_other_2*(1-2*share_own_2);

% other constant
grad(indices(2)) = -weight*(coeff_p(1) + coeff_p_income*income)*share_own_1*share_other_1*(1-2*share_other_1) - ...
          (1-weight)*(coeff_p(2) + coeff_p_income*income)*share_own_2*share_other_2*(1-2*share_other_2);

% coeff_p_income
grad(3) = -weight*((coeff_p(1) + coeff_p_income*income)*income*share_own_1*share_other_1*(p_own+p_other-2*share_own_1*p_own-2*share_other_1*p_other) + income*share_own_1*share_other_1)-...
          (1-weight)*((coeff_p(2) + coeff_p_income*income)*income*share_own_2*share_other_2*(p_own+p_other-2*share_own_2*p_own-2*share_other_2*p_other) + income*share_own_2*share_other_2);

% coeff_p_out
grad(4) = -2*weight*(coeff_p(1) + coeff_p_income*income)*p_out*share_own_1*share_other_1*(1-share_own_1-share_other_1)-...
          2*(1-weight)*(coeff_p(2) + coeff_p_income*income)*p_out*share_own_2*share_other_2*(1-share_own_2-share_other_2);

% coeff_x_outf
grad(5) = -weight*(coeff_p(1) + coeff_p_income*income)*share_own_1*share_other_1*x_outf*(1+1-2*share_own_1-2*share_other_1) -...
          (1-weight)*(coeff_p(2) + coeff_p_income*income)*share_own_2*share_other_2*x_outf*(1+1-2*share_own_2-2*share_other_2);

% coeff_x_usda
if good==1
    
    grad(6) = -weight*(coeff_p(1) + coeff_p_income*income)*share_own_1*share_other_1*(0+x_usda-2*share_own_1*0-2*share_other_1*x_usda) -...
            (1-weight)*(coeff_p(2) + coeff_p_income*income)*share_own_2*share_other_2*(0+x_usda-2*share_own_2*0-2*share_other_2*x_usda);
    
elseif good==2
    
    grad(6) = -weight*(coeff_p(1) + coeff_p_income*income)*share_own_1*share_other_1*(x_usda+0-2*share_own_1*x_usda-2*share_other_1*0) -...
            (1-weight)*(coeff_p(2) + coeff_p_income*income)*share_own_2*share_other_2*(x_usda+0-2*share_own_2*x_usda-2*share_other_2*0);
    
end

% coeff_p
grad(7) = -weight*((coeff_p(1) + coeff_p_income*income)*share_own_1*share_other_1*(p_own+p_other-2*share_own_1*p_own-2*share_other_1*p_other) + ...
                  share_own_1*share_other_1);     % this second term is because coeff_p shows up twice in the own p deriv function

grad(8) = -(1-weight)*((coeff_p(2) + coeff_p_income*income)*share_own_2*share_other_2*(p_own+p_other-2*share_own_2*p_own-2*share_other_2*p_other) + ...
                  share_own_2*share_other_2);     % this second term is because coeff_p shows up twice in the own p deriv function

% weight
grad(9) = -(coeff_p(1) + coeff_p_income*income)*share_own_1*share_other_1 + (coeff_p(2) + coeff_p_income*income)*share_own_2.*share_other_2;

end