%% NB: This requires loading data from Nielsen
%% I am not allowed to post the data, but the paper describes how to obtain the final sample from the raw Nielsen data

load('theta_BCS_initial.mat')

T_new = size(data_trans{1},1);
J = 2;
data_trans_both = [data_trans{1};
                   data_trans{2}];
                                          
nx0 = 1;
               
mat_nohet = [kron(eye(2),ones(T_new,1)) ...
            data_trans_both.p_own.*data_trans_both.income ...
            data_trans_both.p_out_avg ...
            data_trans_both.x_outf ...
            data_trans_both.x_usda_lettuce_2];       % T*J - by - H; first all observations for good 1, then good 2 and so on...
mat_het = [data_trans_both.p_own];     % T*J - by - K; first all observations for good 1, then good 2 and so on...
mat_all = [mat_nohet mat_het];

H = size(mat_nohet,2);      % numer of variables with no r.c.
K = size(mat_het,2);        % number of variables with r.c.

% s_vec = data_trans_both.q_own./(data_trans_both.q_own+data_trans_both.q_other);
% s_vec = log(data_trans_both.q_own)/(max(log(data_trans_both.q_own))+1);
s_vec = data_trans_both.s_own;

% data_trans_both.spot_other = data_trans_both.spot([T_new+1:end 1:T_new]);

IV = [data_trans_both.spot_own   data_trans_both.spot_own.^2 ...
      data_trans_both.spot_other  data_trans_both.spot_other.^2 ...
      (data_trans_both.spot_own - data_trans_both.spot_other).*(data_trans_both.spot_own >= data_trans_both.spot_other) ...
      data_trans_both.spot_own.*data_trans_both.spot_other ...
      data_trans_both.z_out data_trans_both.z_out.^2 ...
      data_trans_both.spot_other.*data_trans_both.income ...
      data_trans_both.income data_trans_both.spot_own.*data_trans_both.income ...
      data_trans_both.x_outf data_trans_both.x_outf.^2 ...
      data_trans_both.x_outf.*data_trans_both.spot_own ...
      data_trans_both.x_outf.*data_trans_both.spot_other ...
      data_trans_both.x_outf.*data_trans_both.income ...
      data_trans_both.x_outf.*data_trans_both.z_out ...
      data_trans_both.x_usda_lettuce_2.*data_trans_both.spot_own ...
      data_trans_both.x_usda_lettuce_2.*data_trans_both.spot_other ...
      data_trans_both.x_usda_lettuce_2.*data_trans_both.income ...
      data_trans_both.x_usda_lettuce_2.*data_trans_both.z_out ...
      data_trans_both.z_own   data_trans_both.z_own.^2 ...
      data_trans_both.z_other  data_trans_both.z_other.^2 ...
      (data_trans_both.z_own - data_trans_both.z_other).*(data_trans_both.z_own >= data_trans_both.z_other) ...
      data_trans_both.z_own.*data_trans_both.z_other ...
      data_trans_both.z_other.*data_trans_both.income ...
      data_trans_both.z_own.*data_trans_both.income ...
      data_trans_both.z_other.*data_trans_both.income.^2 ...
      data_trans_both.z_own.*data_trans_both.income.^2 ...
      data_trans_both.spot_other.*data_trans_both.income.^2 ...
      data_trans_both.spot_own.*data_trans_both.income.^2 ...
      kron(eye(2),ones(T_new,1))];

wmat = inv([IV'*IV]);

% correction for cases where shares are 0
s_vec(s_vec==0) = 10^-5;
s0 = 1.0-sum(reshape(s_vec,[T_new,2]),2)/2;       % s_0 is always zero given the definition of shares; so reduce sum of internal shares (arbitrary)
s0(s0<=0) = 10^-5;

theta_0 = theta_BCS;

constant = theta_0(1:2);
coeff_p_income = theta_0(3);
coeff_p_out = theta_0(4);
coeff_x_outf = theta_0(5);
coeff_x_usda = theta_0(6);
coeff_p = theta_0(7:8);
weight = theta_0(9);

y = log(s_vec) - repmat(log(s0),[2,1]);
mid = mat_all'*IV*wmat*IV';
t = inv(mid*mat_all)*mid*y;
delta_logit = mat_all*t;
csi_logit = delta_logit - (kron(eye(2),ones(T_new,1))*constant'+coeff_p(2).*data_trans_both.p_own+coeff_x_outf.*data_trans_both.x_outf);
theta_logit = [constant coeff_p_income coeff_p_out coeff_x_outf coeff_x_usda coeff_p 0];      % all weight on first type
                              
options = optimset('Algorithm',       'interior-point', ...
            'Display',         'iter-detailed', ...
            'DerivativeCheck', 'off', ...
            'Hessian',         'bfgs', ...
            'GradObj',         'on',  ...
            'MaxFunEvals',     15000, ...
            'TolX',            1e-10, ...
            'GradConstr',      'on',...
            'TolCon',           10^-3,...
            'MaxIter',          1000,...
            'TolFun',           10^-3,...
            'FinDiffRelStep',   10^-1);

obj = @(theta) gmmobj_fixedtol(theta,data_trans_both,mat_nohet,mat_het,s_vec,...
                    IV,T_new,J,K,H,wmat,csi_logit,theta_logit);

lb = -100*ones(size(theta_0,2),1);
ub = 100*ones(size(theta_0,2),1);
lb(end) = 0;
ub(end) = 1;

lb(3:6) = 0;
ub(7:8) = 0;

[theta_BCS,fval_BCS,flag_BCS] = knitromatlab(obj,theta_0,[],[],[],[],lb,ub,[],[],options,'knitro_opt.txt');

[resid_vec,s1,s2] = csi_fixedtol(theta_BCS,data_trans_both,s_vec,T_new,J,csi_logit,theta_logit);
V = var_cov(theta_BCS,s1,s2,mat_nohet,mat_het,resid_vec,IV,T_new,J,K,H,wmat);

se = sqrt(diag(V)/(T_new*J));

theta_BCS

theta_BCS./se'

save('results_BCS.mat')


