clear all
clc

cd('../cvx')
cvx_setup
cd('../application')

%% NB: This requires loading data from Nielsen
%% I am not allowed to post the data, but the paper describes how to obtain the final sample from the raw Nielsen data

%% Set parameters

m_regr = [2 2 2 2 1 1];
m_IV = [2 2 1 1 1 1 1 1];

T1 = size(data_trans{1},1);
T2 = size(data_trans{2},1);
J = 2;

option_symm_p = 'yes';
option_symm_delta = 'no';

%% preparation

B_s{1} = single(zeros(T1,prod(m_regr+1)));
B_IV_final{1} = single(zeros(T1,prod(m_IV+1)));

B_s{2} = single(zeros(T2,prod(m_regr+1)));
B_IV_final{2} = single(zeros(T2,prod(m_IV+1)));

ngrid_s_p = 3;
ngrid_income_nonl = 3;
ngrid_p_out_nonl = 3;

nIV = 3*J+2;
[combos_s_other,combos1_dom_s,combos_p_all,combos_s_all,combos_all,combos_IV,combos_s_nonl,combos_p_nonl,combos_income_nonl,combos_p_out_nonl] = ....
    combinations_nonl_3(m_regr,m_IV,ngrid_s_p,...
    ngrid_income_nonl,ngrid_p_out_nonl,data_trans,nIV);

%% constraints

temp_mat = zeros([m_regr+1]);
temp_mat2 = zeros([m_regr+1,2]);

[ A_ineq_theta,A_eq_theta] = constraint_mat_theta_2(combos1_dom_s,combos_s_all,combos_p_all,combos_all,temp_mat,temp_mat2,m_regr,option_exch,option_symm_delta);
ntheta = size(A_ineq_theta,2);

Aineq = A_ineq_theta;
Aeq = A_eq_theta;

bineq = zeros(size(Aineq,1),1);
beq = zeros(size(Aeq,1),1);

%% estimation

B_IV_final{1} = B_IV_fun_H(data_trans{1},combos_IV,m_IV);     % corresponds to out
B_s{1} = B_s_fun(data_trans{1},combos_all,m_regr);

B_IV_final{2} = B_IV_fun_H(data_trans{2},combos_IV,m_IV);     % corresponds to usda
B_s{2} = B_s_fun(data_trans{2},combos_all,m_regr);

temp = B_IV_final{1}'*B_IV_final{1};
w_mat{1} = inv(temp + 0.05*eye(size(temp)));

temp = B_IV_final{2}'*B_IV_final{2};
w_mat{2} = inv(temp + 0.05*eye(size(temp)));

obj = @(x) estimation_criterion(x,data_trans,w_mat,B_s,B_IV_final);


ub = 100;
lb = -100;


cvx_begin %quiet

cvx_precision default
variable theta_temp(ntheta,1)

x = (reshape(theta_temp,[numel(theta_temp),1]));

minimize( obj(x) )
subject to
Aineq*x <= bineq
x<=ub
x>=lb

cvx_end


theta_sigma = reshape(x(1:ntheta),ntheta,1);

theta_1 = reshape(theta_sigma(1:ntheta/2),[ntheta/2,1]);
theta_2 = reshape(theta_sigma(ntheta/2+1:end),[ntheta/2,1]);

error_1 = (data_trans{1}.x_outf - B_s{1}*theta_1);
error_2 = (data_trans{2}.x_usda_lettuce - B_s{2}*theta_2);

T_1 = size(data_trans{1},1);        % and T_2 = T_1

G_A_1 = 1/T_1*B_IV_final{1}'*B_IV_final{1};
G_A_2 = 1/T_1*B_IV_final{2}'*B_IV_final{2};

S_1 = 1/T_1*B_IV_final{1}'*B_s{1};
S_2 = 1/T_1*B_IV_final{2}'*B_s{2};

Omega_11 = zeros(size(B_IV_final{1},2),size(B_IV_final{1},2));
Omega_22 = zeros(size(B_IV_final{2},2),size(B_IV_final{2},2));
Omega_12 = zeros(size(B_IV_final{1},2),size(B_IV_final{2},2));

for i=1:size(Omega_11,1)
    for j=1:size(Omega_11,1)
        
        Omega_11(i,j) = 1/T_1*sum(B_IV_final{1}(:,i).*B_IV_final{1}(:,j).*(error_1.^2));
        Omega_22(i,j) = 1/T_1*sum(B_IV_final{2}(:,i).*B_IV_final{2}(:,j).*(error_2.^2));
        Omega_12(i,j) = 1/T_1*sum(B_IV_final{1}(:,i).*B_IV_final{2}(:,j).*(error_1.*error_2));
        
    end
    %i
end

G_A = blkdiag(G_A_1,G_A_2);

S = blkdiag(S_1,S_2);

Omega = [Omega_11  Omega_12;
         Omega_12' Omega_22];
     
var_mat = pinv(S'*pinv(G_A)*S)*S'*pinv(G_A)*Omega*pinv(G_A)*S*pinv(S'*pinv(G_A)*S);
     
save('results_NPD.mat')


