function [ grad ] = grad_J_sigma_p_tilde_inv_fun(p1,p2,x_month,x_usda,income,p_out,csi_demeaned,theta)
% returns the gradient of each entry of the INVERSE of J_sigma_p_tilde
% NB: temp refers to the entries of J_sigma_p_tilde (i.e., NOT inverse)
%     temp_grad refers to the entries of the gradient of temp
% NB: each entry of grad corresponds to the gradient of the corresponding
% element of the matrix

temp(1,1) = deriv_BCS_fun_own_p(p1,p2,x_month,x_usda,income,p_out,csi_demeaned,theta,1);
temp(1,2) = deriv_BCS_fun_cross_p(p1,p2,x_month,x_usda,income,p_out,csi_demeaned,theta,1);

temp(2,1) = deriv_BCS_fun_cross_p(p2,p1,x_month,x_usda,income,p_out,csi_demeaned([2 1]),theta,2);
temp(2,2) = deriv_BCS_fun_own_p(p2,p1,x_month,x_usda,income,p_out,csi_demeaned([2 1]),theta,2);

temp_grad{1,1} = deriv_BCS_fun_own_p_grad(p1,p2,x_month,x_usda,income,p_out,csi_demeaned,theta,1);
temp_grad{1,2} = deriv_BCS_fun_cross_p_grad(p1,p2,x_month,x_usda,income,p_out,csi_demeaned,theta,1);

temp_grad{2,1} = deriv_BCS_fun_cross_p_grad(p2,p1,x_month,x_usda,income,p_out,csi_demeaned([2 1]),theta,2);
temp_grad{2,2} = deriv_BCS_fun_own_p_grad(p2,p1,x_month,x_usda,income,p_out,csi_demeaned([2 1]),theta,2);

determ = det(temp);

determ_grad = temp_grad{1,1}*temp(2,2) + temp(1,1)*temp_grad{2,2} - (temp_grad{1,2}*temp(2,1) + temp(1,2)*temp_grad{2,1});

% apply formula for inverse of a 2-by-2 matrix
grad = zeros(2,2,length(theta));

grad(1,1,:) = 1/determ^2*(temp_grad{2,2}*determ - temp(2,2)*determ_grad);
grad(1,2,:) = 1/determ^2*(-temp_grad{1,2}*determ + temp(1,2)*determ_grad);
grad(2,1,:) = 1/determ^2*(-temp_grad{2,1}*determ + temp(2,1)*determ_grad);
grad(2,2,:) = 1/determ^2*(temp_grad{1,1}*determ - temp(1,1)*determ_grad);

end