%% NB: This requires loading data from Nielsen
%% I am not allowed to post the data, but the paper describes how to obtain the final sample from the raw Nielsen data

tax_perc = 0.25;
tax_1 = [mean(data_trans{1}.p_own)*tax_perc 0];
tax_2 = [0 mean(data_trans{2}.p_own)*tax_perc];

tol_NPD = 0.1;
increment_grad = 10^-4;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This part produces the results for the BCS parametric model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First we need to load the results from the "estimation_BCS.m" script
load('results_BCS.mat')

nx0 = 10;

T_1 = size(data_trans{1},1);

p_delta_1_BCS = zeros(T_1,1);
p_delta_2_BCS = zeros(T_1,1);
p_perc_1_BCS = zeros(T_1,1);
p_perc_2_BCS = zeros(T_1,1);
der_delta_1_BCS = zeros(length(theta_BCS),T_1);
der_delta_2_BCS = zeros(length(theta_BCS),T_1);
der_perc_1_BCS = zeros(length(theta_BCS),T_1);
der_perc_2_BCS = zeros(length(theta_BCS),T_1);
p_delta_1_BCS_se = zeros(T_1,1);
p_delta_2_BCS_se = zeros(T_1,1);
p_perc_1_BCS_se = zeros(T_1,1);
p_perc_2_BCS_se = zeros(T_1,1);
flag_mc = zeros(T_1,1);
mc_BCS = zeros(T_1,2);

for i=1:T_1
    
    %% prep
    
    s_1_bar = data_trans{1}.s_own(i);
    s_2_bar = data_trans{2}.s_own(i);
    p_1_bar = data_trans{1}.p_own(i);
    p_2_bar = data_trans{2}.p_own(i);
    p_1_bar_nonn = data_trans{1}.p_own_nonn(i);
    p_2_bar_nonn = data_trans{2}.p_own_nonn(i);
    income_bar = data_trans{1}.income(i);
    x_outf_bar = data_trans{1}.x_outf(i);
    x_usda_bar = data_trans{1}.x_usda_lettuce(i);
    p_out_bar = data_trans{1}.p_out_avg(i);
    
    data_bar.p_own = [p_1_bar;
        p_2_bar];
    
    data_bar.p_other = [p_2_bar;
        p_1_bar];
    
    data_bar.income = income_bar*ones(2,1);
    data_bar.p_out_avg = p_out_bar*ones(2,1);
    data_bar.x_outf = x_outf_bar*ones(2,1);
    data_bar.x_usda_lettuce_2 = x_usda_bar*ones(2,1);
    data_bar.x_usda_lettuce_2(1) = 0;
    
    s_vec_bar = [s_1_bar;
        s_2_bar];
    
    csi_bar = csi_fixedtol(theta_BCS,data_bar,s_vec_bar,1,J,[0 0]',theta_logit);
    
    mc_BCS(i,:) = mc_fun_BCS(p_1_bar,p_2_bar,[p_1_bar_nonn p_2_bar_nonn]',s_1_bar,s_2_bar,x_outf_bar,x_usda_bar,income_bar,p_out_bar,csi_bar,...
                            data,option_prices,theta_BCS);
    
    if any(mc_BCS(i,:)<0)
        flag_mc(i) = 1; 
    end
    
    %% good 1
    
    p0 = [p_1_bar p_2_bar];
    [p_eq,p_eq_norm,flag] = eq_price_est_foc_fun_multi_tax_BCS(mc_BCS(i,1),mc_BCS(i,2),csi_bar(1),csi_bar(2),x_outf_bar,x_usda_bar,p0(1),p0(2),...
        income_bar,p_out_bar,data.p_own,theta_BCS,option_prices,nx0,[0 0]);
    
    p0 = p_eq_norm;
    [p_eq_cf,p_eq_cf_norm,flag_cf] = eq_price_est_foc_fun_multi_tax_BCS(mc_BCS(i,1),mc_BCS(i,2),csi_bar(1),csi_bar(2),x_outf_bar,x_usda_bar,p0(1),p0(2),...
        income_bar,p_out_bar,data.p_own,theta_BCS,option_prices,nx0,tax_1);
    
    p0 = [p_1_bar p_2_bar];
    mat = grad_theta_num_p_multi_tax_BCS(x_outf_bar,x_usda_bar,income_bar,p_out_bar,csi_bar,p_eq,mc_BCS(i,:),theta_BCS,increment_grad,data,option_prices,p0,nx0,[0 0]);
    
    p0 = p_eq_cf_norm;
    mat_cf = grad_theta_num_p_multi_tax_BCS(x_outf_bar,x_usda_bar,income_bar,p_out_bar,csi_bar,p_eq_cf,mc_BCS(i,:),theta_BCS,increment_grad,data,option_prices,p0,nx0,tax_1);
    
    p_tax_cf_nonn = normalize_p_inv_fun(p_eq_cf_norm(1) + tax_1(1),data.p_own,option_prices);
    
    p_delta_1_BCS(i) = p_tax_cf_nonn - p_eq(1);
    
    % delta method
    
    mat_final = mat_cf - mat;
    der_delta_1_BCS(:,i) = mat_final(1:9,1);
    der_perc_1_BCS(:,i) = mat_final(1:9,1)/(p_1_bar_nonn*tax_perc);
    
    V1_BCS = mat_final(1:9,1)'*V*mat_final(1:9,1);
    p_delta_1_BCS_se(i) = sqrt(V1_BCS/T_1);
    
    p_perc_1_BCS(i) = p_delta_1_BCS(i)/(p_1_bar_nonn*tax_perc);
    p_perc_1_BCS_se(i) = p_delta_1_BCS_se(i)/(p_1_bar_nonn*tax_perc);
    
    %% good 2
        
    p0 = [p_1_bar p_2_bar];
    [p_eq,p_eq_norm,flag] = eq_price_est_foc_fun_multi_tax_BCS(mc_BCS(i,1),mc_BCS(i,2),csi_bar(1),csi_bar(2),x_outf_bar,x_usda_bar,p0(1),p0(2),...
        income_bar,p_out_bar,data.p_own,theta_BCS,option_prices,nx0,[0 0]);
    
    p0 = p_eq_norm;
    [p_eq_cf,p_eq_cf_norm,flag_cf] = eq_price_est_foc_fun_multi_tax_BCS(mc_BCS(i,1),mc_BCS(i,2),csi_bar(1),csi_bar(2),x_outf_bar,x_usda_bar,p0(1),p0(2),...
        income_bar,p_out_bar,data.p_own,theta_BCS,option_prices,nx0,tax_2);
    
    p0 = [p_1_bar p_2_bar];
    mat = grad_theta_num_p_multi_tax_BCS(x_outf_bar,x_usda_bar,income_bar,p_out_bar,csi_bar,p_eq,mc_BCS(i,:),theta_BCS,...
                                    increment_grad,data,option_prices,p0,nx0,[0 0]);
    
    p0 = p_eq_cf_norm;
    mat_cf = grad_theta_num_p_multi_tax_BCS(x_outf_bar,x_usda_bar,income_bar,p_out_bar,csi_bar,p_eq_cf,mc_BCS(i,:),theta_BCS,...
        increment_grad,data,option_prices,p0,nx0,tax_2);
    
    p_tax_cf_nonn = normalize_p_inv_fun(p_eq_cf_norm(2) + tax_2(2),data.p_own,option_prices);
    
    p_delta_2_BCS(i) = p_tax_cf_nonn - p_eq(2);
    
    % delta method
    
    mat_final = mat_cf - mat;
    der_delta_2_BCS(:,i) = mat_final(1:9,2);
    der_perc_2_BCS(:,i) = mat_final(1:9,2)/(p_2_bar_nonn*tax_perc);
    
    V2_BCS = mat_final(1:9,2)'*V*mat_final(1:9,2);
    p_delta_2_BCS_se(i) = sqrt(V2_BCS/T_1);
    
    p_perc_2_BCS(i) = p_delta_2_BCS(i)/(p_2_bar_nonn*tax_perc);
    p_perc_2_BCS_se(i) = p_delta_2_BCS_se(i)/(p_2_bar_nonn*tax_perc);
    
    i
    
end

p_delta_1_BCS_mean = mean(p_delta_1_BCS);
p_delta_2_BCS_mean = mean(p_delta_2_BCS);
p_perc_1_BCS_mean = mean(p_perc_1_BCS);
p_perc_2_BCS_mean = mean(p_perc_2_BCS);

der_delta_1_BCS_mean = mean(der_delta_1_BCS,2);
der_delta_2_BCS_mean = mean(der_delta_2_BCS,2);
der_perc_1_BCS_mean = mean(der_perc_1_BCS,2);
der_perc_2_BCS_mean = mean(der_perc_2_BCS,2);

% good 1

V_BCS = der_delta_1_BCS_mean'*V*der_delta_1_BCS_mean;
p_delta_1_BCS_mean_se = sqrt(V_BCS/T_1);

V_BCS = der_perc_1_BCS_mean'*V*der_perc_1_BCS_mean;
p_perc_1_BCS_mean_se = sqrt(V_BCS/T_1);


% good 2

V_BCS = der_delta_2_BCS_mean'*V*der_delta_2_BCS_mean;
p_delta_2_BCS_mean_se = sqrt(V_BCS/T_1);

V_BCS = der_perc_2_BCS_mean'*V*der_perc_2_BCS_mean;
p_perc_2_BCS_mean_se = sqrt(V_BCS/T_1);


%% get confidence intervals

cv = 1.96;

display('********************** PERCENTAGE OF P ********************************')

display('***good 1 -- BCS***')
[p_perc_1_BCS_mean-cv*p_perc_1_BCS_mean_se,p_perc_1_BCS_mean+cv*p_perc_1_BCS_mean_se]

display('***good 2 -- BCS***')
[p_perc_2_BCS_mean-cv*p_perc_2_BCS_mean_se,p_perc_2_BCS_mean+cv*p_perc_2_BCS_mean_se]


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% This part produces the results for the NPD parametric model
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% First we need to load the results from the "estimation_NPD.m" script
load('results_NPD.mat')

p_delta_1_NPD = zeros(T_1,1);
p_delta_2_NPD = zeros(T_1,1);
p_perc_1_NPD = zeros(T_1,1);
p_perc_2_NPD = zeros(T_1,1);
der_delta_1_NPD = zeros(ntheta,T_1);
der_delta_2_NPD = zeros(ntheta,T_1);
der_perc_1_NPD = zeros(ntheta,T_1);
der_perc_2_NPD = zeros(ntheta,T_1);
p_delta_1_NPD_se = zeros(T_1,1);
p_delta_2_NPD_se = zeros(T_1,1);
p_perc_1_NPD_se = zeros(T_1,1);
p_perc_2_NPD_se = zeros(T_1,1);
flag_mc = zeros(T_1,1);
mc_NPD = zeros(T_1,2);

for i=1:T_1
    
    %% prep
           
    s_1_bar = data_trans{1}.s_own(i);
    s_2_bar = data_trans{2}.s_own(i);
    p_1_bar = data_trans{1}.p_own(i);
    p_2_bar = data_trans{2}.p_own(i);
    p_1_bar_nonn = data_trans{1}.p_own_nonn(i);
    p_2_bar_nonn = data_trans{2}.p_own_nonn(i);
    income_bar = data_trans{1}.income(i);
    p_out_bar = data_trans{1}.p_out_avg(i);
    
    delta_1_bar = fun_sigmainv_est([s_1_bar s_2_bar p_1_bar p_2_bar income_bar p_out_bar],theta_sigma,combos_all,m_regr,1);
    delta_2_bar = fun_sigmainv_est([s_2_bar s_1_bar p_2_bar p_1_bar income_bar p_out_bar],theta_sigma,combos_all,m_regr,2);
    delta_bar = [delta_1_bar delta_2_bar];
    
    %% get estimates and s.e.
    
    mc_NPD(i,:) = mc_fun_normaliz_NPD(theta_sigma,[p_1_bar p_2_bar],[p_1_bar_nonn p_2_bar_nonn],[s_1_bar s_2_bar],...
                    income_bar,p_out_bar,m_regr,combos_all,data,option_prices)';
    
    if any(mc_NPD(i,:)<0)
        flag_mc(i) = 1; 
    end
    
    % good 1
    
    [p_eq,p_eq_norm,flag] = eq_price_est_foc_fun_multi_tax_NPD(mc_NPD(i,:),delta_bar,income_bar,p_out_bar,data.p_own,...
                                    theta_sigma,combos_all,m_regr,option_prices,[p_1_bar p_2_bar],nx0,[0 0]);
                                
    grad_p = grad_theta_num_prices_multinew_tax_NPD(theta_sigma,delta_bar,income_bar,p_out_bar,mc_NPD(i,:),p_eq,m_regr,combos_all,...
                                                data,option_prices,increment_grad,[p_1_bar p_2_bar],nx0,[0 0],tol_NPD);                            
                                
    [p_eq_cf,p_eq_cf_norm,flag_cf] = eq_price_est_foc_fun_multi_tax_NPD(mc_NPD(i,:),delta_bar,income_bar,p_out_bar,data.p_own,...
        theta_sigma,combos_all,m_regr,option_prices,p_eq_norm,nx0,tax_1);
    
    grad_p_cf = grad_theta_num_prices_multinew_tax_NPD(theta_sigma,delta_bar,income_bar,p_out_bar,mc_NPD(i,:),p_eq_cf,m_regr,combos_all,...
        data,option_prices,increment_grad,p_eq_cf_norm,nx0,tax_1,tol_NPD);
    
    p_tax_cf_nonn = normalize_p_inv_fun(p_eq_cf_norm(1) + tax_1(1),data.p_own,option_prices);
    
    p_delta_1_NPD(i) = p_tax_cf_nonn - p_eq(1);
    p_perc_1_NPD(i) = p_delta_1_NPD(i)/(p_1_bar_nonn*tax_perc);
    
    mat = grad_p_cf - grad_p;
    der_delta_1_NPD(:,i) = mat(:,1);
    
    V = der_delta_1_NPD(:,i)'*var_mat*der_delta_1_NPD(:,i);
    p_delta_1_NPD_se(i) = sqrt(V/T_1);
    p_perc_1_NPD_se(i) = p_delta_1_NPD_se(i)/(p_1_bar_nonn*tax_perc);
    p_delta_1_NPD_se(i)/(p_1_bar_nonn*tax_perc)
    der_perc_1_NPD(:,i) = der_delta_1_NPD(:,i)/(p_1_bar_nonn*tax_perc);
    
    % good 2
    
    [p_eq,p_eq_norm,flag] = eq_price_est_foc_fun_multi_tax_NPD(mc_NPD(i,:),delta_bar,income_bar,p_out_bar,data.p_own,...
        theta_sigma,combos_all,m_regr,option_prices,[p_1_bar p_2_bar],nx0,[0 0]);
        
    [p_eq_cf,p_eq_cf_norm,flag_cf] = eq_price_est_foc_fun_multi_tax_NPD(mc_NPD(i,:),delta_bar,income_bar,p_out_bar,data.p_own,...
                                    theta_sigma,combos_all,m_regr,option_prices,p_eq_cf_norm,nx0,tax_2);
    
    grad_p_cf = grad_theta_num_prices_multinew_tax_NPD(theta_sigma,delta_bar,income_bar,p_out_bar,mc_NPD(i,:),p_eq_cf,m_regr,combos_all,...
                                                   data,option_prices,increment_grad,p_eq_cf_norm,nx0,tax_2,tol_NPD);                            
                                
    p_tax_cf_nonn = normalize_p_inv_fun(p_eq_cf_norm(2) + tax_2(2),data.p_own,option_prices);                            
                                
    p_delta_2_NPD(i) = p_tax_cf_nonn - p_eq(2);
    p_perc_2_NPD(i) = p_delta_2_NPD(i)/(p_2_bar_nonn*tax_perc);
    
    mat = grad_p_cf - grad_p;
    der_delta_2_NPD(:,i) = mat(:,2);
    
    V = der_delta_2_NPD(:,i)'*var_mat*der_delta_2_NPD(:,i);
    p_delta_2_NPD_se(i) = sqrt(V/T_1);
    p_perc_2_NPD_se(i) = p_delta_2_NPD_se(i)/(p_2_bar_nonn*tax_perc);
    p_delta_2_NPD_se(i)/(p_2_bar_nonn*tax_perc)
    der_perc_2_NPD(:,i) = der_delta_2_NPD(:,i)/(p_2_bar_nonn*tax_perc);
    
    i
    
end

p_delta_1_NPD_mean = mean(p_delta_1_NPD);
p_delta_2_NPD_mean = mean(p_delta_2_NPD);
p_perc_1_NPD_mean = mean(p_perc_1_NPD);
p_perc_2_NPD_mean = mean(p_perc_2_NPD);

der_delta_1_NPD_mean = mean(der_delta_1_NPD,2);
der_delta_2_NPD_mean = mean(der_delta_2_NPD,2);
der_perc_1_NPD_mean = mean(der_perc_1_NPD,2);
der_perc_2_NPD_mean = mean(der_perc_2_NPD,2);

% good 1

V = der_delta_1_NPD_mean'*var_mat(1:ntheta,1:ntheta)*der_delta_1_NPD_mean;
p_delta_1_NPD_mean_se = sqrt(V/T_1);

V = der_perc_1_NPD_mean'*var_mat(1:ntheta,1:ntheta)*der_perc_1_NPD_mean;
p_perc_1_NPD_mean_se = sqrt(V/T_1);


% good 2

V = der_delta_2_NPD_mean'*var_mat(1:ntheta,1:ntheta)*der_delta_2_NPD_mean;
p_delta_2_NPD_mean_se = sqrt(V/T_1);

V = der_perc_2_NPD_mean'*var_mat(1:ntheta,1:ntheta)*der_perc_2_NPD_mean;
p_perc_2_NPD_mean_se = sqrt(V/T_1);


%% get confidence intervals

display('********************** PERCENTAGE OF P ********************************')

display('***good 1 -- NPD***')
[p_perc_1_NPD_mean-cv*p_perc_1_NPD_mean_se,p_perc_1_NPD_mean+cv*p_perc_1_NPD_mean_se]

display('***good 2 -- NPD***')
[p_perc_2_NPD_mean-cv*p_perc_2_NPD_mean_se,p_perc_2_NPD_mean+cv*p_perc_2_NPD_mean_se]


