%% This code produces Figure 2 in the paper

%% Note that the code requires being able to call the function "knitromatlab," which is part of the KNITRO software
% Alternatively, one can estimate the BLP model using fmincon from MATLAB
% (this requires using the function
% fun_estimation_BLP_fixedtol_fmincon_2const in lieu of
% fun_estimation_BLP_fixedtol_knitro_2const)

%% This installs cvx, which is available for download at http://cvxr.com/cvx/download/

clear all
clc

cd('../cvx')
cvx_setup
cd('../montecarlos')

%% NPD parameters

m_regr = [6 6 6 6];
m_IV = 6;

ngrid_invert_sigma = 150;

%% DGP parameters

constant = 1;        % mean of csi shocks
alpha_mu = -0.75;
beta_mu = 0.75;
alpha_sigma = 0.25;
beta_sigma = 0;
eta = 2;
eps = 0.1;

T = 3000;
J = 2;
ns = 250;
K = 1;

nMC = 200;

N = 1000;

rc_draw(1,:) = alpha_mu + alpha_sigma*randn(N,1);
rc_draw(2,:) = beta_mu + beta_sigma*randn(N,1);

theta1_true = [constant constant alpha_mu beta_mu];
theta2_true = [alpha_sigma beta_sigma]';
theta2_0 = theta2_true;

%% initial values for BLP and NPD routines

theta1_BLP = zeros(length(theta1_true),nMC);
theta2_BLP = zeros(length(theta2_true),nMC);
fval_BLP = zeros(1,nMC);
flag_BLP = zeros(1,nMC);

theta_NPD = zeros(2*prod(m_regr+1),nMC);
fval_NPD = zeros(1,nMC);
flag_NPD = cell(1,nMC);

%% estimation routine

for ss=1:nMC
        
    data =  generate_data_simple_BLP(T,rc_draw,eta,constant);
    T_new = size(data,1);
        
    % BLP estimation
        
    [theta1_BLP(:,ss),theta2_BLP(:,ss),fval_BLP(ss),flag_BLP(ss)] = fun_estimation_BLP_fixedtol_fmincon_2const(data,ns,K,J,theta2_0,T_new,10);
%     [theta1_BLP(:,ss),theta2_BLP(:,ss),fval_BLP(ss),flag_BLP(ss)] = fun_estimation_BLP_fixedtol_knitro_2const(data,ns,K,J,theta2_0,T_new);
    
    % NPD estimation
        
    option_exch = 'no';        % 'yes': demand is the same for the two goods
    option_symm = 'yes';
    data_norm = data;
        
    [data_norm.p, df_dp] = fnorm_p(data.p,data.p,eps);
    data_norm.x = fnorm_p(data.x,data.x,eps);
    [data_trans] = transform_data(data_norm,J);
    
    nIV = size(data.x,2)+size(data.z,2);       % use same IVs as in BLP procedure
    
    [theta_NPD(:,ss),fval_NPD(ss),flag_NPD{ss}] = fun_estimation_NPD_2(data_trans,m_regr,m_IV,nIV,option_exch,option_symm);
        
    ss
    
end

%% Compute and plot elasticities

data =  generate_data_simple_BLP(T,rc_draw,eta,constant);

theta_BLP = [theta1_BLP;
             theta2_BLP];
         
normal_draw = randn(2,N);

ntheta = size(theta_NPD,1);

ngrid_p = 15;
grid_p = linspace(quantile(data.p(:),0.05),quantile(data.p(:),0.95),ngrid_p);

ela_own_BLP = zeros(ngrid_p,nMC);
ela_own_NPD = zeros(ngrid_p,nMC);
ela_own_true = zeros(ngrid_p,1);
ela_cross_BLP = zeros(ngrid_p,nMC);
ela_cross_NPD = zeros(ngrid_p,nMC);
ela_cross_true = zeros(ngrid_p,1);

p_other = quantile(data.p(:),0.50);
% shares = quantile(data.s,0.50);
x = quantile(data.x,0.50);
csi = [constant constant];
csi_demeaned = [0 0];        % want to let BLP estimate the mean of csi

temp = cell(4,1);
for i=1:4
    
    temp{i} = 1:m_regr(4-i+1)+1;
    
end
combos_all = allcomb(temp{:});     
combos_all = combos_all(:,[4 3 2 1]);

increment = 0.001;

for i=1:ngrid_p
        
    share1 = share_fun_inattention(grid_p(i),p_other,x,csi,rc_draw,1);
    share1_incr1 = share_fun_inattention(grid_p(i)+increment,p_other,x,csi,rc_draw,1);
    share1_incr2 = share_fun_inattention(grid_p(i),p_other+increment,x,csi,rc_draw,1);
    
    ela_own_true(i) = (share1_incr1-share1)/increment*grid_p(i)/share1;
    ela_cross_true(i) = (share1_incr2-share1)/increment*p_other/share1;
    
    share2 = share_fun_inattention(p_other,grid_p(i),x([2 1]),csi([2 1]),rc_draw,2);
        
    parfor ss=1:nMC
        
        if all(theta_BLP(:,ss)>=-19.99) && all(theta_BLP(:,ss)<=19.99)      % rule out samples where the lb/ub are binding
            
            ela_own_BLP(i,ss) = elasticity_BLP_fun_own_p_2const(grid_p(i),p_other,x,csi_demeaned,normal_draw,theta_BLP(:,ss));
            ela_cross_BLP(i,ss) = elasticity_BLP_fun_cross_p_2const(grid_p(i),p_other,x,csi_demeaned,normal_draw,theta_BLP(:,ss));
            
        end
        
    end
        
    [ p_own_norm, df_dp ] = fnorm_p(grid_p(i),data.p,eps);
    p_other_norm = fnorm_p(p_other,data.p,eps);     % df_dp is the same as in line above
    
    delta1 = csi(1) + x(1);
    delta2 = csi(2) + x(2);
    
    parfor ss=1:nMC
        
        theta1 = theta_NPD(1:ntheta/2,ss);
        theta2 = theta_NPD(ntheta/2+1:ntheta,ss);
        
%         [share1,share2] = fun_sigma_est(p_own_norm,p_other_norm,delta1,delta2,theta1,theta2,combos_all,m_regr,ngrid_invert_sigma);
        
        J_sigmainv_s = jacobian_sigma_inv_s_2(theta_NPD(:,ss),[share1 share2],[p_own_norm p_other_norm],combos_all,m_regr);
        J_sigmainv_p = jacobian_sigma_inv_p_2(theta_NPD(:,ss),[share1 share2],[p_own_norm p_other_norm],combos_all,m_regr);
        J_norm_p = [df_dp 0;
                    0     df_dp];
        
        J_sigma_p = -inv(J_sigmainv_s)*J_sigmainv_p*J_norm_p;     % Jacobian of function with output (s1 s2)               
 
        ela_own_NPD(i,ss) = J_sigma_p(1,1)*grid_p(i)/share1;         % own price elasticity of good 1
        ela_cross_NPD(i,ss) = J_sigma_p(1,2)*p_other/share1;         % cross price elasticity of good 1 wrt p2
        
    end  
    i
end

q025_own_BLP = quantile(ela_own_BLP',0.025,1);
q975_own_BLP = quantile(ela_own_BLP',0.975,1);
q025_own_NPD = quantile(ela_own_NPD',0.025,1);
q975_own_NPD = quantile(ela_own_NPD',0.975,1);
q025_cross_BLP = quantile(ela_cross_BLP',0.025,1);
q975_cross_BLP = quantile(ela_cross_BLP',0.975,1);
q025_cross_NPD = quantile(ela_cross_NPD',0.025,1);
q975_cross_NPD = quantile(ela_cross_NPD',0.975,1);


figure(1)
pl0 = plot(grid_p,ela_own_true','-k','linewidth',1.25);
hold on
pl1 = line_fewer_markers(grid_p,q025_own_BLP,5,'--^','color','red','markersize',3);
hold on
pl2 = line_fewer_markers(grid_p,q975_own_BLP,5,'--^','color','red','markersize',3);
hold on
pl3 = line_fewer_markers(grid_p,q025_own_NPD,5,'--*','color','blue','markersize',5);
hold on
pl4 = line_fewer_markers(grid_p,q975_own_NPD,5,'--*','color','blue','markersize',5);
legend([pl0 pl1 pl3],'True','BLP 95% CI','NPD 95% CI')
hold off
xlabel('Price')
ylabel('Own Elasticity')
ax = gca;
outerpos = ax.OuterPosition;
ti = ax.TightInset; 
left = outerpos(1) + ti(1);
bottom = outerpos(2) + ti(2);
ax_width = outerpos(3) - ti(1) - ti(3);
ax_height = outerpos(4) - ti(2) - ti(4);
ax.Position = [left bottom ax_width ax_height];
fig = gcf;
fig.PaperPositionMode = 'auto'
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
axis([min(grid_p)-0.25   max(grid_p)+0.25   -4.5   0])


figure(2)
pl0 = plot(grid_p,ela_cross_true','-k','linewidth',1.25);
hold on
pl1 = line_fewer_markers(grid_p,q025_cross_BLP,5,'--^','color','red','markersize',3);
hold on
pl2 = line_fewer_markers(grid_p,q975_cross_BLP,5,'--^','color','red','markersize',3);
hold on
pl3 = line_fewer_markers(grid_p,q025_cross_NPD,5,'--*','color','blue','markersize',5);
hold on
pl4 = line_fewer_markers(grid_p,q975_cross_NPD,5,'--*','color','blue','markersize',5);
legend([pl0 pl1 pl3],'True','BLP 95% CI','NPD 95% CI')
hold off
xlabel('Price')
ylabel('Cross Elasticity')
ax = gca;
outerpos = ax.OuterPosition;
ti = ax.TightInset; 
left = outerpos(1) + ti(1);
bottom = outerpos(2) + ti(2);
ax_width = outerpos(3) - ti(1) - ti(3);
ax_height = outerpos(4) - ti(2) - ti(4);
ax.Position = [left bottom ax_width ax_height];
fig = gcf;
fig.PaperPositionMode = 'auto'
fig_pos = fig.PaperPosition;
fig.PaperSize = [fig_pos(3) fig_pos(4)];
axis([min(grid_p)-0.25   max(grid_p)+0.25   0   2])




