%% This code produces Table 2 in the paper

%% This installs cvx, which is available for download at http://cvxr.com/cvx/download/

clear all
clc

cd('../cvx')
cvx_setup
cd('../montecarlos')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Degree = 6
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% NPD parameters

m_regr = [2 2 1 1];
m_IV = 2;

ngrid_invert_sigma = 150;

%% DGP parameters

alpha_mu = -1;
beta_mu = 1;

alpha_sigma = 0.15;
beta_sigma = 0;

gamma = 2;           % the higher gamma, the higher the marginal costs
constant = 1;        % mean of csi shocks

T = 3000;
J = 2;
K = 1;

nMC = 200;

N = 1000;

normal_draw = randn(2,N);

rc_draw(1,:) = alpha_mu + alpha_sigma*normal_draw(1,:);
rc_draw(2,:) = beta_mu + beta_sigma*normal_draw(2,:);

theta1_true = [constant alpha_mu beta_mu];
theta2_true = [alpha_sigma beta_sigma]';
theta2_0 = theta2_true;

%% initial values for NPD routine

theta_NPD = zeros(2*prod(m_regr+1),nMC);
fval_NPD = zeros(1,nMC);
flag_NPD = cell(1,nMC);

%% estimation routine

for ss=1:nMC
        
    data = generate_data_simple_2z(T,rc_draw,gamma,J,constant);
    T_new = size(data,1);
        
    % NPD estimation
        
    option_exch = 'yes';        % 'yes': demand is the same for the two goods
    option_symm = 'no';
    data_norm = data;
    eps = 0.1;
    
    [data_norm.p, df_dp] = fnorm_p(data.p,data.p,eps);
    [data_norm.x] = fnorm_p(data.x,data.x,eps);
    [data_trans] = transform_data(data_norm,J);
    
    nIV = size(data.x,2)+size(data.z,2);
    
    [theta_NPD(:,ss),fval_NPD(ss),flag_NPD{ss}] = fun_estimation_NPD_2(data_trans,m_regr,m_IV,nIV,option_exch,option_symm);
            
    ss
    
end

%% Obtain bias and variance for median elasticities

data = generate_data_simple_2z(T,rc_draw,gamma,J,constant);

ngrid_s_nonl = [2 2];
ngrid_p_nonl = [2 2];

[combos_s2,combos_p2,combos_s1_s2,combos_p1_p2,combos_all,combos_IV,nonlin_s,nonlin_p] = combinations_2(m_regr,m_IV,ngrid_s_nonl,ngrid_p_nonl,data_trans,nIV);

T_temp = 500;

data_norm = data;
eps = 0.1;

ntheta = size(theta_NPD,1);

increment = 0.001;

ela_own_NPD = zeros(T_temp,nMC);
ela_own_true = zeros(T_temp,1);
ela_cross_NPD = zeros(T_temp,nMC);
ela_cross_true = zeros(T_temp,1);

for i=1:T_temp
        
    p = data.p(i,:)';
    x = data.x(i,:)';
    csi = data.csi(i,:)';
    p_other = p(2:J)';
    
    shares = share_fun_J(p,x,csi,rc_draw,J);
                
    p_incr1 = [p(1)+increment p_other]';
    shares_incr1 = share_fun_J(p_incr1,x,csi,rc_draw,J);
    
    p_incr2 = [p(1) p_other(1)+increment p_other(2:J-1)]';
    shares_incr2 = share_fun_J(p_incr2,x,csi,rc_draw,J);
    
    ela_own_true(i) = (shares_incr1(1)-shares(1))/increment*p(1)/shares(1);
    ela_cross_true(i) = (shares_incr2(1)-shares(1))/increment*p_other(1)/shares(1);
    
    share_other = shares(2);
        
    [ p_own_norm, df_dp ] = fnorm_p(p(1),data.p,eps);
    p_other_norm = fnorm_p(p(2:J),data.p,eps);     % df_dp is the same as in line above
    
    for ss=1:nMC
        
        theta1 = theta_NPD(1:ntheta/2,ss);
        theta2 = theta_NPD(ntheta/2+1:ntheta,ss);
                
        J_sigmainv_s = jacobian_sigma_inv_s_2(theta_NPD(:,ss),shares',[p_own_norm p_other_norm],combos_all,m_regr);
        J_sigmainv_p = jacobian_sigma_inv_p_2(theta_NPD(:,ss),shares',[p_own_norm p_other_norm],combos_all,m_regr);
        J_norm_p = [df_dp 0;
                    0     df_dp];
        
        J_sigma_p = -inv(J_sigmainv_s)*J_sigmainv_p*J_norm_p;     % Jacobian of function with output (s1 s2)               
 
        ela_own_NPD(i,ss) = J_sigma_p(1,1)*p(1)/shares(1);         % own price elasticity of good 1
        ela_cross_NPD(i,ss) = J_sigma_p(1,2)*p(2)/shares(1);         % cross price elasticity of good 1 wrt p2
        
    end
    
end

% estimates

ela_own_NPD_median = median(ela_own_NPD,1);
ela_cross_NPD_median = median(ela_cross_NPD,1);

% truth

ela_own_true_median = median(ela_own_true,1);
ela_cross_true_median = median(ela_cross_true,1);

% MSE Median
display('************************* Median NPD -- Degree = 6 *********************************')

bias_own_NPD = nanmean(ela_own_NPD_median)-ela_own_true_median
bias_cross_NPD = nanmedian(ela_cross_NPD_median)-ela_cross_true_median

stdev_own_NPD = nanstd(ela_own_NPD_median)
stdev_cross_NPD = nanstd(ela_cross_NPD_median)

MSE_own_median_NPD = bias_own_NPD^2 + stdev_own_NPD^2
MSE_cross_median_NPD = bias_cross_NPD^2 + stdev_cross_NPD^2

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Degree = 8
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% NPD parameters

m_regr = [2 2 2 2];
m_IV = 2;

ngrid_invert_sigma = 150;

%% DGP parameters

alpha_mu = -1;
beta_mu = 1;

alpha_sigma = 0.15;
beta_sigma = 0;

gamma = 2;           % the higher gamma, the higher the marginal costs
constant = 1;        % mean of csi shocks

T = 3000;
J = 2;
K = 1;

nMC = 200;

N = 1000;

normal_draw = randn(2,N);

rc_draw(1,:) = alpha_mu + alpha_sigma*normal_draw(1,:);
rc_draw(2,:) = beta_mu + beta_sigma*normal_draw(2,:);

theta1_true = [constant alpha_mu beta_mu];
theta2_true = [alpha_sigma beta_sigma]';
theta2_0 = theta2_true;

%% initial values for NPD routine

theta_NPD = zeros(2*prod(m_regr+1),nMC);
fval_NPD = zeros(1,nMC);
flag_NPD = cell(1,nMC);

%% estimation routine

for ss=1:nMC
        
    data = generate_data_simple_2z(T,rc_draw,gamma,J,constant);
    T_new = size(data,1);
        
    % NPD estimation
        
    option_exch = 'yes';        % 'yes': demand is the same for the two goods
    option_symm = 'no';
    data_norm = data;
    eps = 0.1;
    
    [data_norm.p, df_dp] = fnorm_p(data.p,data.p,eps);
    [data_norm.x] = fnorm_p(data.x,data.x,eps);
    [data_trans] = transform_data(data_norm,J);
    
    nIV = size(data.x,2)+size(data.z,2);
    
    [theta_NPD(:,ss),fval_NPD(ss),flag_NPD{ss}] = fun_estimation_NPD_2(data_trans,m_regr,m_IV,nIV,option_exch,option_symm);
            
    ss
    
end

%% Obtain bias and variance for median elasticities

data = generate_data_simple_2z(T,rc_draw,gamma,J,constant);

ngrid_s_nonl = [2 2];
ngrid_p_nonl = [2 2];

[combos_s2,combos_p2,combos_s1_s2,combos_p1_p2,combos_all,combos_IV,nonlin_s,nonlin_p] = combinations_2(m_regr,m_IV,ngrid_s_nonl,ngrid_p_nonl,data_trans,nIV);

T_temp = 500;

data_norm = data;
eps = 0.1;

ntheta = size(theta_NPD,1);

increment = 0.001;

ela_own_NPD = zeros(T_temp,nMC);
ela_own_true = zeros(T_temp,1);
ela_cross_NPD = zeros(T_temp,nMC);
ela_cross_true = zeros(T_temp,1);

for i=1:T_temp
        
    p = data.p(i,:)';
    x = data.x(i,:)';
    csi = data.csi(i,:)';
    p_other = p(2:J)';
    
    shares = share_fun_J(p,x,csi,rc_draw,J);
                
    p_incr1 = [p(1)+increment p_other]';
    shares_incr1 = share_fun_J(p_incr1,x,csi,rc_draw,J);
    
    p_incr2 = [p(1) p_other(1)+increment p_other(2:J-1)]';
    shares_incr2 = share_fun_J(p_incr2,x,csi,rc_draw,J);
    
    ela_own_true(i) = (shares_incr1(1)-shares(1))/increment*p(1)/shares(1);
    ela_cross_true(i) = (shares_incr2(1)-shares(1))/increment*p_other(1)/shares(1);
    
    share_other = shares(2);
        
    [ p_own_norm, df_dp ] = fnorm_p(p(1),data.p,eps);
    p_other_norm = fnorm_p(p(2:J),data.p,eps);     % df_dp is the same as in line above
    
    for ss=1:nMC
        
        theta1 = theta_NPD(1:ntheta/2,ss);
        theta2 = theta_NPD(ntheta/2+1:ntheta,ss);
                
        J_sigmainv_s = jacobian_sigma_inv_s_2(theta_NPD(:,ss),shares',[p_own_norm p_other_norm],combos_all,m_regr);
        J_sigmainv_p = jacobian_sigma_inv_p_2(theta_NPD(:,ss),shares',[p_own_norm p_other_norm],combos_all,m_regr);
        J_norm_p = [df_dp 0;
                    0     df_dp];
        
        J_sigma_p = -inv(J_sigmainv_s)*J_sigmainv_p*J_norm_p;     % Jacobian of function with output (s1 s2)               
 
        ela_own_NPD(i,ss) = J_sigma_p(1,1)*p(1)/shares(1);         % own price elasticity of good 1
        ela_cross_NPD(i,ss) = J_sigma_p(1,2)*p(2)/shares(1);         % cross price elasticity of good 1 wrt p2
        
    end
    
end

% estimates

ela_own_NPD_median = median(ela_own_NPD,1);
ela_cross_NPD_median = median(ela_cross_NPD,1);

% truth

ela_own_true_median = median(ela_own_true,1);
ela_cross_true_median = median(ela_cross_true,1);

% MSE Median
display('************************* Median NPD -- Degree = 8 *********************************')

bias_own_NPD = nanmean(ela_own_NPD_median)-ela_own_true_median
bias_cross_NPD = nanmedian(ela_cross_NPD_median)-ela_cross_true_median

stdev_own_NPD = nanstd(ela_own_NPD_median)
stdev_cross_NPD = nanstd(ela_cross_NPD_median)

MSE_own_median_NPD = bias_own_NPD^2 + stdev_own_NPD^2
MSE_cross_median_NPD = bias_cross_NPD^2 + stdev_cross_NPD^2


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Degree = 12
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% NPD parameters

m_regr = [3 3 3 3];
m_IV = 3;

ngrid_invert_sigma = 150;

%% DGP parameters

alpha_mu = -1;
beta_mu = 1;

alpha_sigma = 0.15;
beta_sigma = 0;

gamma = 2;           % the higher gamma, the higher the marginal costs
constant = 1;        % mean of csi shocks

T = 3000;
J = 2;
K = 1;

nMC = 200;

N = 1000;

normal_draw = randn(2,N);

rc_draw(1,:) = alpha_mu + alpha_sigma*normal_draw(1,:);
rc_draw(2,:) = beta_mu + beta_sigma*normal_draw(2,:);

theta1_true = [constant alpha_mu beta_mu];
theta2_true = [alpha_sigma beta_sigma]';
theta2_0 = theta2_true;

%% initial values for NPD routine

theta_NPD = zeros(2*prod(m_regr+1),nMC);
fval_NPD = zeros(1,nMC);
flag_NPD = cell(1,nMC);

%% estimation routine

for ss=1:nMC
        
    data = generate_data_simple_2z(T,rc_draw,gamma,J,constant);
    T_new = size(data,1);
        
    % NPD estimation
        
    option_exch = 'yes';        % 'yes': demand is the same for the two goods
    option_symm = 'no';
    data_norm = data;
    eps = 0.1;
    
    [data_norm.p, df_dp] = fnorm_p(data.p,data.p,eps);
    [data_norm.x] = fnorm_p(data.x,data.x,eps);
    [data_trans] = transform_data(data_norm,J);
    
    nIV = size(data.x,2)+size(data.z,2);
    
    [theta_NPD(:,ss),fval_NPD(ss),flag_NPD{ss}] = fun_estimation_NPD_2(data_trans,m_regr,m_IV,nIV,option_exch,option_symm);
            
    ss
    
end

%% Obtain bias and variance for median elasticities

data = generate_data_simple_2z(T,rc_draw,gamma,J,constant);

ngrid_s_nonl = [2 2];
ngrid_p_nonl = [2 2];

[combos_s2,combos_p2,combos_s1_s2,combos_p1_p2,combos_all,combos_IV,nonlin_s,nonlin_p] = combinations_2(m_regr,m_IV,ngrid_s_nonl,ngrid_p_nonl,data_trans,nIV);

T_temp = 500;

data_norm = data;
eps = 0.1;

ntheta = size(theta_NPD,1);

increment = 0.001;

ela_own_NPD = zeros(T_temp,nMC);
ela_own_true = zeros(T_temp,1);
ela_cross_NPD = zeros(T_temp,nMC);
ela_cross_true = zeros(T_temp,1);

for i=1:T_temp
        
    p = data.p(i,:)';
    x = data.x(i,:)';
    csi = data.csi(i,:)';
    p_other = p(2:J)';
    
    shares = share_fun_J(p,x,csi,rc_draw,J);
                
    p_incr1 = [p(1)+increment p_other]';
    shares_incr1 = share_fun_J(p_incr1,x,csi,rc_draw,J);
    
    p_incr2 = [p(1) p_other(1)+increment p_other(2:J-1)]';
    shares_incr2 = share_fun_J(p_incr2,x,csi,rc_draw,J);
    
    ela_own_true(i) = (shares_incr1(1)-shares(1))/increment*p(1)/shares(1);
    ela_cross_true(i) = (shares_incr2(1)-shares(1))/increment*p_other(1)/shares(1);
    
    share_other = shares(2);
        
    [ p_own_norm, df_dp ] = fnorm_p(p(1),data.p,eps);
    p_other_norm = fnorm_p(p(2:J),data.p,eps);     % df_dp is the same as in line above
    
    for ss=1:nMC
        
        theta1 = theta_NPD(1:ntheta/2,ss);
        theta2 = theta_NPD(ntheta/2+1:ntheta,ss);
                
        J_sigmainv_s = jacobian_sigma_inv_s_2(theta_NPD(:,ss),shares',[p_own_norm p_other_norm],combos_all,m_regr);
        J_sigmainv_p = jacobian_sigma_inv_p_2(theta_NPD(:,ss),shares',[p_own_norm p_other_norm],combos_all,m_regr);
        J_norm_p = [df_dp 0;
                    0     df_dp];
        
        J_sigma_p = -inv(J_sigmainv_s)*J_sigmainv_p*J_norm_p;     % Jacobian of function with output (s1 s2)               
 
        ela_own_NPD(i,ss) = J_sigma_p(1,1)*p(1)/shares(1);         % own price elasticity of good 1
        ela_cross_NPD(i,ss) = J_sigma_p(1,2)*p(2)/shares(1);         % cross price elasticity of good 1 wrt p2
        
    end
    
end

% estimates

ela_own_NPD_median = median(ela_own_NPD,1);
ela_cross_NPD_median = median(ela_cross_NPD,1);

% truth

ela_own_true_median = median(ela_own_true,1);
ela_cross_true_median = median(ela_cross_true,1);

% MSE Median
display('************************* Median NPD -- Degree = 12 *********************************')

bias_own_NPD = nanmean(ela_own_NPD_median)-ela_own_true_median
bias_cross_NPD = nanmedian(ela_cross_NPD_median)-ela_cross_true_median

stdev_own_NPD = nanstd(ela_own_NPD_median)
stdev_cross_NPD = nanstd(ela_cross_NPD_median)

MSE_own_median_NPD = bias_own_NPD^2 + stdev_own_NPD^2
MSE_cross_median_NPD = bias_cross_NPD^2 + stdev_cross_NPD^2


