%% This code produces Table 3 in the paper

%% This installs cvx, which is available for download at http://cvxr.com/cvx/download/

clear all
clc

cd('../cvx')
cvx_setup
cd('../montecarlos')

%% DGP parameters

T = 3000;

nMC = 200;

N = 5000;
tol1 = 0.01;
tol2 = 0.05;
ngrid_p = 100;
p_max = 6;     % with higher p_max, don't get convergence sometimes

gamma = 2;           % the higher gamma, the higher the marginal costs
constant = 1;        % mean of csi shocks

nx0 = 5;

temp = [0.5 3];

rc_draw(1,:) = -temp(randi(2,[1,N]));
rc_draw(2,:) = -rc_draw(1,:);
% NB: I am using this order:
% 1st r.c. is for p
% 2nd r.c. is for x

ns = 100;
K = 1;      % number of exogenous characteristics with random coefficient
H = 2;      % numer of variables entering linearly through the mean utility, besides dummies

J = 3;  % change this to J=5 or J=10 to populate the table

%% NPD prep

m_regr = 2*ones(1,J);
m_IV = 2*ones(1,J+1);

[combos_all_unique,combos_IV,combos_perms] = combinations_2_s(m_regr,J,m_IV);

combos_IV_small = combos_IV(sum(combos_IV-1,2)<=m_IV(1),:);

% constraints

option_exch = 'yes';        % 'yes': demand is the same for the two goods
option_symm = 'no';
eps = 0.1;

[ A_ineq_theta,b_ineq_theta,A_eq_theta,b_eq_theta ] = constraint_mat_theta_2_s(combos_all_unique,m_regr,J,option_symm);
ntheta = size(A_ineq_theta,2);

Aineq = A_ineq_theta;
Aeq = A_eq_theta;
   
bineq = zeros(size(Aineq,1),1);
beq = zeros(size(Aeq,1),1);

%% initial values for BLP and NPD routines

theta2_0 = [1/4 1/4]';

theta1_BLP = zeros(3,nMC);
theta2_BLP = zeros(2,nMC);
fval_BLP = zeros(1,nMC);
flag_BLP = zeros(1,nMC);

theta_NPD = zeros(ntheta,nMC);
p_coeff_NPD = zeros(1,nMC);

%% estimation routine

for ss=1:nMC
        
    data = generate_data_simple_2z_J(T,rc_draw,gamma,J,constant);
    T_new = size(data,1);
               
    % BLP estimation
        
    [theta1_BLP(:,ss),theta2_BLP(:,ss),fval_BLP(ss),flag_BLP(ss)] = fun_estimation_BLP_fixedtol_fmincon_J(data,ns,K,J,theta2_0,T_new,nx0);
    
    % NPD estimation
        
    option_exch = 'yes';        % 'yes': demand is the same for the two goods
    option_symm = 'no';
    data_norm = data;
    eps = 0.1;
    
    [data_norm.p, df_dp] = fnorm_p(data.p,data.p,eps);
    [data_norm.x] = fnorm_p(data.x,data.x,eps);
    [data_trans] = transform_data(data_norm,J);
    
    nIV = size(data.x,2)+size(data.z,2);       % use same IVs as in BLP procedure
    
    [theta_NPD(:,ss),p_coeff_NPD(ss)] = estimation_NPD_cvx_2_s(data_trans,J,T,m_regr,m_IV,Aineq,Aeq,bineq,beq,...
        combos_all_unique,combos_IV_small,option_symm,ntheta);
        
    ss
    
end

%% Compute elasticities

data = generate_data_simple_2z_J(T,rc_draw,gamma,J,constant);

T_temp = 500;

data_norm = data;
eps = 0.1;

N = 1000;
normal_draw = randn(2,N);

csi_demeaned = zeros(1,J);

% [data_norm.p, df_dp] = fnorm_p(data.p,data.p,eps);
% [data_norm.x] = fnorm_p(data.x,data.x,eps);
% [data_trans] = transform_data(data_norm,J);

ntheta = size(theta_NPD,1);

increment = 0.001;

theta_BLP = [theta1_BLP;
            theta2_BLP];

ela_own_BLP = zeros(T_temp,nMC);
ela_own_NPD = zeros(T_temp,nMC);
ela_own_true = zeros(T_temp,1);
ela_cross_BLP = zeros(T_temp,nMC);
ela_cross_NPD = zeros(T_temp,nMC);
ela_cross_true = zeros(T_temp,1);

for i=1:T_temp
        
    p = data.p(i,:)';
    x = data.x(i,:)';
    csi = data.csi(i,:)';
    p_other = p(2:J)';
    
    shares = share_fun_J(p,x,csi,rc_draw,J);
                
    p_incr1 = [p(1)+increment p_other]';
    shares_incr1 = share_fun_J(p_incr1,x,csi,rc_draw,J);
    
    p_incr2 = [p(1) p_other(1)+increment p_other(2:J-1)]';
    shares_incr2 = share_fun_J(p_incr2,x,csi,rc_draw,J);
    
    ela_own_true(i) = (shares_incr1(1)-shares(1))/increment*p(1)/shares(1);
    ela_cross_true(i) = (shares_incr2(1)-shares(1))/increment*p_other(1)/shares(1);
    
    share_other = shares(2);
        
    parfor ss=1:nMC
        
        ela_own_BLP(i,ss) = elasticity_BLP_fun_own_p_J(p(1),p(2:J),x,csi_demeaned',normal_draw,theta_BLP(:,ss),J);
        ela_cross_BLP(i,ss) = elasticity_BLP_fun_cross_p_J(p(1),p(2:J),x,csi_demeaned',normal_draw,theta_BLP(:,ss),J);
        
    end
        
    [ p_own_norm, df_dp ] = fnorm_p(p(1),data.p,eps);
    p_other_norm = fnorm_p(p(2:J),data.p,eps);
    
    parfor ss=1:nMC
        %tic
        theta = theta_NPD(1:ntheta,ss);
                
        J_sigmainv_s = jacobian_sigma_inv_s_J(theta,shares',m_regr,J,combos_all_unique,combos_perms);
        J_norm_p = eye(J)*df_dp*p_coeff_NPD(ss);
        
        J_sigma_p = inv(J_sigmainv_s)*J_norm_p;     % Jacobian of function with output (s1 s2)               
 
        ela_own_NPD(i,ss) = J_sigma_p(1,1)*p(1)/shares(1);         % own price elasticity of good 1
        ela_cross_NPD(i,ss) = J_sigma_p(1,2)*p(2)/shares(1);         % cross price elasticity of good 1 wrt p2
        %toc
    end  
    i
end

% estimates

ela_own_BLP_median = median(ela_own_BLP,1);
ela_cross_BLP_median = median(ela_cross_BLP,1);

ela_own_NPD_median = median(ela_own_NPD,1);
ela_cross_NPD_median = median(ela_cross_NPD,1);

% truth

ela_own_true_median = median(ela_own_true,1);
ela_cross_true_median = median(ela_cross_true,1);

display('************************* Median NPD *********************************')

bias_own_NPD = nanmean(ela_own_NPD_median)-ela_own_true_median
bias_cross_NPD = nanmedian(ela_cross_NPD_median)-ela_cross_true_median

stdev_own_NPD = nanstd(ela_own_NPD_median)
stdev_cross_NPD = nanstd(ela_cross_NPD_median)

MSE_own_median_NPD = bias_own_NPD^2 + stdev_own_NPD^2
MSE_cross_median_NPD = bias_cross_NPD^2 + stdev_cross_NPD^2


display('************************* Median BLP*********************************')

bias_own_BLP = mean(ela_own_BLP_median)-ela_own_true_median
bias_cross_BLP = median(ela_cross_BLP_median)-ela_cross_true_median

stdev_own_BLP = std(ela_own_BLP_median)
stdev_cross_BLP = std(ela_cross_BLP_median)

MSE_own_median_BLP = bias_own_BLP^2 + stdev_own_BLP^2
MSE_cross_median_BLP = bias_cross_BLP^2 + stdev_cross_BLP^2
